"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es.array.concat.js");

var _isEvaluable = _interopRequireDefault(require("./isEvaluable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Evaluates a value.
 *
 * Dispatches to the `evaluate` method if it exists. If an evaluate method returns
 * a value that is also evaluate, this method will evaluate that value as
 * well.
 *
 * @private
 * @function
 * @since v0.3.0
 * @category config.util
 * @param {*} any The value to evaluate.
 * @returns {*} The evaluated value.
 * @example
 *
 * evaluate('foo')
 * // => 'foo'
 *
 * evaluate({
 *  evaluate: () => 'bar'
 * })
 * //=> bar
 *
 * evaluate({
 *   evaluate: () => ({
 *     evaluate: () => 'bar'
 *   })
 * })
 * //=> bar
 */
var evaluate = function evaluate(any) {
  if ((0, _isEvaluable.default)(any)) {
    for (var _len = arguments.length, rest = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      rest[_key - 1] = arguments[_key];
    }

    return evaluate.apply(void 0, [any.evaluate.apply(any, rest)].concat(rest));
  }

  return any;
};

var _default = evaluate;
exports.default = _default;
//# sourceMappingURL=evaluate.js.map