"use strict";

require("core-js/modules/es.object.keys.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.object.get-own-property-descriptor.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/es.object.get-own-property-descriptors.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Iterator = require("../constants/Iterator");

var _reflectOwnKeys = _interopRequireDefault(require("./reflectOwnKeys"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var iterAt = function iterAt(index, keys, object) {
  if (index < keys.length && index >= 0) {
    var key = keys[index];
    return {
      done: false,
      kdx: key,
      key: key,
      value: object[key]
    };
  }

  return {
    done: true
  };
};

var prevIterAt = function prevIterAt(index, keys, object) {
  if (index < keys.length && index >= 0) {
    var key = keys[index];
    return {
      done: false,
      kdx: key,
      key: key,
      value: object[key]
    };
  }
};
/**
 * Returns iterator for an object's keys and values.
 *
 * Note, iterates over object's own keys and symbols
 *
 * @private
 * @function
 * @since v0.1.0
 * @category lang.util
 * @param {*} object The array object to create an iterator for.
 * @return {Iterator} A new iterator for the given object's keys and values
 * @example
 *
 * const iter = objectToIterator({
 *   write: 'more',
 *   tests: 'asap',
 *   [Symbol('like')]: 'now'
 * })
 * //=> {
 * //   next: () => ({
 * //     value: *,
 * //     done: boolean,
 * //     kdx: string,
 * //     key: string
 * //   })
 * // }
 *
 * iter.next()
 * //=> { value: 'more', key: 'write', kdx: 'write', done: false }
 *
 * iter.next()
 * //=> { value: 'asap', key: 'tests', kdx: 'tests', done: false }
 *
 * iter.next()
 * //=> { value: 'now', key: Symbol('like'), kdx: Symbol('like'), done: false }
 *
 * iter.next()
 * //=> { done: true }
 */


var objectToIterator = function objectToIterator(object) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _Iterator.START;
  var keys = (0, _reflectOwnKeys.default)(object);
  var index = 0;
  var lastIndex;

  if (start === _Iterator.END) {
    index = keys.length;
  }

  return {
    getKey: function getKey() {
      return keys[lastIndex];
    },
    next: function next() {
      var iter = iterAt(index, keys, object);
      var prev = prevIterAt(index - 1, keys, object);

      if (index < keys.length) {
        lastIndex = index;
        index += 1;
      }

      return _objectSpread(_objectSpread({}, iter), {}, {
        prev: prev
      });
    },
    previous: function previous() {
      var iter = iterAt(index - 1, keys, object);
      var prev = prevIterAt(index, keys, object);

      if (index >= 0) {
        lastIndex = index - 1;
        index -= 1;
      }

      return _objectSpread(_objectSpread({}, iter), {}, {
        prev: prev
      });
    }
  };
};

var _default = objectToIterator;
exports.default = _default;
//# sourceMappingURL=objectToIterator.js.map